import {Globalize, PrintDocument, escapeHtml} from '@mescius/wijmo';
import {CellType, HeadersVisibility} from '@mescius/wijmo.grid';
import {FlexGridPdfConverter, ScaleMode} from '@mescius/wijmo.grid.pdf';

function renderRow(panel, r) {
    let tr = '';
    const row = panel.rows[r];

    if (row.renderSize > 0) {
        tr += '<tr>';
        for (let c = 0; c < panel.columns.length; c += 1) {
            const col = panel.columns[c];
            if (col.renderSize > 0) {
                // get cell style, content
                // prettier-ignore
                const style = `width: ${col.renderSize}px; text-align: ${col.getAlignment()}; padding-right: 6px;`;
                let content = panel.getCellData(r, c, true);
                if (!row.isContentHtml && !col.isContentHtml) content = escapeHtml(content);

                // add cell to row
                if (panel.cellType === CellType.ColumnHeader) {
                    tr += `<th style="${style}">${content}</th>`;
                } else {
                    // show boolean values as checkboxes
                    const raw = panel.getCellData(r, c, false);
                    if (raw === true) {
                        content = '&#9745;';
                    } else if (raw === false) {
                        content = '&#9744;';
                    }

                    tr += `<td style="${style}">${content}</td>`;
                }
            }
        }
        tr += '</tr>';
    }
    return tr;
}

function scaleY(value, min, max) {
    return max > min ? 100 - Math.round((value - min) / (max - min) * 100) : 0;
}

// generate sparklines as SVG
function getSparklines(data) {
    const min = Math.min(...data);
    const max = Math.max(...data);
    const svg = [];
    let x1 = 0;
    let y1 = scaleY(data[0], min, max);
    for (let i = 1; i < data.length; i += 1) {
        const x2 = Math.round(i / (data.length - 1) * 100);
        const y2 = scaleY(data[i], min, max);
        svg.push(`<line x1="${x1}%" y1="${y1}%" x2="${x2}%" y2="${y2}%" />`);
        x1 = x2;
        y1 = y2;
    }
    return `<svg><g>${svg.join()}</g></svg>`;
}

function formatDynamicCell(item, col, history, flare) {
    const tradeHistory = [...item.history].splice(0, 10); // get last 10 elements
    tradeHistory.reverse();
    const hist = tradeHistory.map(element => element[history]);
    const change = `${Globalize.format(item.chg / item.open * 100, 'n1')}%`;
    const glyph = item.chg > +0.001 ? 'up' : item.chg < -0.001 ? 'down' : 'circle'; // eslint-disable-line
    const spark = getSparklines(hist); // sparklines
    const dir = glyph === 'circle' ? 'none' : glyph; // change direction
    const flareDir = flare ? dir : 'none'; // // flare direction

    // cell template
    return [
        `<div class="ticker chg-${dir} flare-${flareDir}">`,
        `<div class="chg">${change}</div>`,
        `<div class="glyph"><span class="wj-glyph-${glyph}"/></div>`,
        `<div class="spark">${spark}</div>`,
        `</div>`,
    ].join('');
}

function formatTechnicalIndicator(item, binding) {
    const value = Math.round(item[binding]);
    const format = id =>
        ({
            [-2]: '強い売り',
            [-1]: '売り',
            0: 'ニュートラル',
            1: '買い',
            2: '強い買い',
        }[id] || 'Unknown');

    return [
        `<div class="indicator ${value > 0 && 'buy'} ${value < 0 && 'sell'}">`,
        `${format(value)}`,
        `</div>`,
    ].join('');
}

function formatPerformanceIndicator(item, binding) {
    const value = Math.round(item[binding]);
    return [
        `<div class="indicator ${value > 0 && 'buy'} ${value < 0 && 'sell'}">`,
        `${value}%`,
        `</div>`,
    ].join('');
}

/**
 * Export the current view to a PDF file.
 * @param flex - Wijmo FlexGrid
 * @param filteredData - "soft refreshed" data
 */
function renderTable(flex, filteredData) {
    // header
    const head = [
        '<thead>',
        flex.columnHeaders.rows.map((row, index) => renderRow(flex.columnHeaders, index)).join(''), // prettier-ignore
        '</thead>',
    ].join('');
    // body
    const body = [
        '<tbody>',
      // flex.rows.map((row, index) => renderRow(flex.cells, index)).join(''),
      // substitute row data with "soft refreshed" ones
        filteredData.map((row, index) => {
          flex.cells.rows[index]._data = row;
          return renderRow(flex.cells, index);
        }).join(''),
        '</tbody>',
    ].join('');
    // table
    return [
        '<table>',
        `${flex.headersVisibility && HeadersVisibility.Column ? head : ''}`,
        `${body}`,
        '</table>',
    ].join('');
}

/**
 * Print the current view.
 * @param grid - Wijmo FlexGrid
 * @param filteredData - "soft refreshed" data
 */
export function print(grid, filteredData) {
    const doc = new PrintDocument({
        title: 'PrintDocument',
        copyCss: true,
    });
    const content = [
        `<h1>印刷サンプル</h1>`,
        `<p>この文書は<b>PrintDocument</b>クラスを使用して作成されました。</p>`,
        `<p>これはテーブルとして描画されたFlexGridです。</p>`,
      // send "soft refreshed" data to renderTable() method
        `${renderTable(grid, filteredData)}`,
    ].join('');

    doc.append(content); // add some simple text
    doc.print(); // print the document
}

/**
 * Export the current view to a PDF file.
 * @param grid - Wijmo FlexGrid
 * @param filteredData - "soft refreshed" data
 */
export function pdf(grid, filteredData) {
    // update source data
    grid.itemsSource = filteredData;
    FlexGridPdfConverter.export(grid, 'FlexGrid.pdf', {
        embeddedFonts: [{ source: `https://${window.location.host}/wijmo/demos/fonts/ipaexg.ttf`, name: 'ipaexg' }],
        scaleMode: ScaleMode.PageWidth,
        documentOptions: {
            compress: true,
            header: {declarative: {text: '\t&[Page] of &[Pages]'}},
            footer: {declarative: {text: '\t&[Page] of &[Pages]'}},
            info: {author: 'C1', title: 'FlexGrid'},
        },
        styles: {
            cellStyle: { font: { family: 'ipaexg' },
            backgroundColor: '#ffffff', borderColor: '#c6c6c6'},
            altCellStyle: {backgroundColor: '#f9f9f9'},
            groupCellStyle: {backgroundColor: '#dddddd'},
            headerCellStyle: {backgroundColor: '#eaeaea'},
        },
    });
}

/**
 * Custom cell painting.
 * @param item - data item
 * @param col - column
 * @param flare - custom painting flag
 */
export function formatCell(item, col, flare) {
    let result;
    switch (col.binding) {
        case 'technicalMinutes15':
            result = formatTechnicalIndicator(item, 'technicalMinutes15');
            break;
        case 'technicalMinutes30':
            result = formatTechnicalIndicator(item, 'technicalMinutes15');
            break;
        case 'technicalHourly':
            result = formatTechnicalIndicator(item, 'technicalHourly');
            break;
        case 'technicalWeekly':
            result = formatTechnicalIndicator(item, 'technicalWeekly');
            break;
        case 'technicalMonthly':
            result = formatTechnicalIndicator(item, 'technicalMonthly');
            break;
        case 'performanceDaily':
            result = formatPerformanceIndicator(item, 'performanceDaily');
            break;
        case 'performanceWeek':
            result = formatPerformanceIndicator(item, 'performanceWeek');
            break;
        case 'performanceMonth':
            result = formatPerformanceIndicator(item, 'performanceMonth');
            break;
        case 'performanceYtd':
            result = formatPerformanceIndicator(item, 'performanceYtd');
            break;
        case 'performanceYear':
            result = formatPerformanceIndicator(item, 'performanceYear');
            break;
        case 'performanceYear3':
            result = formatPerformanceIndicator(item, 'performanceYear3');
            break;
        case 'chg':
            result = formatDynamicCell(item, col, 'close', flare);
            break;
        default:
            result = Globalize.format(item[col.binding], col.format);
            break;
    }
    return result;
}
